Object subclass: #EnchantingPeripheral
	instanceVariableNames: 'name concreteType portIndex '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Enchanting-Configuration'!
!EnchantingPeripheral commentStamp: '<historical>' prior: 0!
The EnchantingPeripheral is the abstract base class for all peripherals that may be configured for devices.  Specific classes for sensors and motors come off of this.!


!EnchantingPeripheral methodsFor: 'accessing' stamp: 'CWB 12/15/2010 22:02'!
name

	^ name.! !

!EnchantingPeripheral methodsFor: 'accessing' stamp: 'CWB 12/17/2010 02:53'!
portIndex
"A number indicating which port this device is on."

	^ portIndex.! !

!EnchantingPeripheral methodsFor: 'accessing' stamp: 'CWB 12/17/2010 02:53'!
portIndex: aPortIndex
"Sets a number indicating which port this device is on."

	portIndex _ aPortIndex.! !


!EnchantingPeripheral methodsFor: 'conversion' stamp: 'CWB 10/26/2012 18:51'!
createConfigBlock
"A peripheral should create a config block and fill it in with pertinent information"

	| block |

	block _ self class createGenericConfigBlock.
	block initialName: name.
	block concreteType: concreteType.
	^ block.

! !

!EnchantingPeripheral methodsFor: 'conversion' stamp: 'CWB 11/22/2010 20:31'!
takeSettingsFrom: aConfigBlockMorph
"With a morph of the appropriate class, this gets the necessary settings from the configuration block"

	name _ aConfigBlockMorph currentName.
	concreteType _ aConfigBlockMorph concreteType.
	^ self.! !


!EnchantingPeripheral methodsFor: 'exporting' stamp: 'CWB 12/17/2010 02:15'!
emitInitializationCodeOn: aStream for: anExporter
"Outputs the code to initialize the various devices."

	self subclassResponsibility.! !


!EnchantingPeripheral methodsFor: 'initialization' stamp: 'CWB 12/17/2010 02:52'!
initialize

	super initialize.
	name _ self class baseName.
	concreteType _ nil.
	portIndex _ nil.! !


!EnchantingPeripheral methodsFor: 'object i/o' stamp: 'CWB 12/10/2010 07:01'!
fieldsVersion

	^ 1
! !

!EnchantingPeripheral methodsFor: 'object i/o' stamp: 'CWB 12/17/2010 02:52'!
initFieldsFrom: anObjStream version: classVersion

	"super initFieldsFrom: anObjStream version: classVersion."
	self initFieldsNamed: #(
		name
		concreteType
		portIndex
	) from: anObjStream.! !

!EnchantingPeripheral methodsFor: 'object i/o' stamp: 'CWB 12/17/2010 02:52'!
storeFieldsOn: anObjStream

	"super storeFieldsOn: anObjStream."
	self storeFieldsNamed: #(
		name
		concreteType
		portIndex
	) on: anObjStream.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

EnchantingPeripheral class
	instanceVariableNames: ''!

!EnchantingPeripheral class methodsFor: 'accessing' stamp: 'CWB 11/20/2010 20:07'!
baseName
"Returns the basic name for devices of this type, such as 'light sensor' or 'motor'"

	self subclassResponsibility.! !

!EnchantingPeripheral class methodsFor: 'accessing' stamp: 'CWB 12/18/2010 12:10'!
concreteTypes
"Returns a list of physical devices that implement the functions of this class."

	self subclassResponsibility.! !


!EnchantingPeripheral class methodsFor: 'block specs' stamp: 'CWB 10/26/2012 06:56'!
configBlockIsNameless
"Does this config block allow you to name the item?  Almost all do."

	^ false.! !

!EnchantingPeripheral class methodsFor: 'block specs' stamp: 'CWB 11/20/2010 20:04'!
configBlockSpec
"Provide the english specification for the configuration block"

	self subclassResponsibility.! !

!EnchantingPeripheral class methodsFor: 'block specs' stamp: 'CWB 12/16/2010 12:53'!
defaultArgs

	"Return the default arguments."

	"This basic implementation looks for the name in the spec and fills it in."

	| spec argSpecs defaults |

	spec _ self configBlockSpec.
	argSpecs _ (CommandBlockMorph parseCommandSpec: spec)
		select:  [:s | CommandBlockMorph isArgSpec: s].

	defaults _ OrderedCollection new.
	argSpecs do: [:arg |
		| type val |
		type _ arg second.	"One character that defines the type"
		val _ ''.
		(type = $s) ifTrue: [ val _ self baseName ].
		defaults add: val.
	].

	^ defaults. 
		! !

!EnchantingPeripheral class methodsFor: 'block specs' stamp: 'CWB 1/3/2011 09:59'!
obsoleteBlocks
"Returns a list of blocks that used to be provided by this block but have been deprecated and should not be used in the future.  These blocks will still load from files, but can not be freshly created in new programs, and need not be translated"

	^ #()	"by default, we assume there are no obsolete blocks"
! !

!EnchantingPeripheral class methodsFor: 'block specs' stamp: 'CWB 12/11/2010 22:06'!
providesBlocks
"Returns a list of block specifications to be used in the main user interface if there is any device configured that can use these blocks."

	self subclassResponsibility.! !


!EnchantingPeripheral class methodsFor: 'conversion' stamp: 'CWB 10/26/2012 07:05'!
createGenericConfigBlock
"Returns a ConfigurationBlockMorph for the specified class"

	| englishSpec blockSpec argPermutation defaultArgs |

	englishSpec _ self configBlockSpec.
	blockSpec _ ScratchTranslator translationFor: englishSpec.
	argPermutation _ CommandBlockMorph argPermutationForSpec: englishSpec withTranslation: blockSpec.
	defaultArgs _ self defaultArgs.
		
	^ ConfigurationBlockMorph new
		argPermutation: argPermutation;
		commandSpec: blockSpec;
		englishCodeSpec: englishSpec;
		defaultArgs: defaultArgs;
		peripheralClass: self;
		nameless: self configBlockIsNameless.

	"Things that I did not set -- color, selector, receiver"! !


!EnchantingPeripheral class methodsFor: 'dropdown lists' stamp: 'CWB 12/16/2010 20:47'!
customListNumber: aNumber
"Peripherals can return custom lists (or nil)."

	self subclassResponsibility.! !


!EnchantingPeripheral class methodsFor: 'equivalence' stamp: 'CWB 12/11/2010 22:03'!
actsAs: aClass
"Returns true if this device acts as another device."
"For example, a color sensor may also act as a light sensor."

"In the general case, any given item acts as all the things it is derived from"

	^ self includesBehavior: aClass.! !

!EnchantingPeripheral class methodsFor: 'equivalence' stamp: 'CWB 12/18/2010 17:09'!
providesBlocksUsedBy: aCollectionOfClasses
"Returns true if this class provides blocks that are used by any of the supplied classes"

	aCollectionOfClasses detect: [:c | c actsAs: self] ifNone: [ ^ false ].
	^ true.! !
