Morph subclass: #ElementsPaletteFrameMorph
	instanceVariableNames: 'header palette editorFrame currentClass '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Elements-UI'!

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/5/2009 20:30'!
acceptDroppingMorph: aMorph event: evt
	"This message is sent when a morph is dropped onto a morph that has agreed to accept the dropped morph by responding 'true' to the wantsDroppedMorph:Event: message. This default implementation just adds the given morph to the receiver."

	aMorph delete
! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/5/2009 20:14'!
arrangePanes
	header position: self position;
		width: self width;
		arrangePanes.
	palette position: self left @ header bottom;
		extent: self width @ (self height - header height)! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 01:25'!
buildPanes
	header _ PaletteFrameHeadMorph new frame: self.
	palette _ ElementsScrollFrameMorph new
				color: Color gray;
				borderColor: Color gray.
	self
		addMorph: header;
		addMorph: palette.! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/7/2009 22:06'!
class: aClass
	currentClass _ aClass.
	header adjustToClass: currentClass.
	self refresh! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 21:17'!
classMenu
	|menu|
	menu _ MenuMorph new defaultTarget: self.
	menu add: 'create a subclass...' action: #createSubclass.
	menu add: 'remove a class...' action: #removeClass.
	^menu! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 21:40'!
classVarMenu
	|menu|
	menu _ MenuMorph new defaultTarget: self.
	menu add: 'add a class variable...' action: #createClassVar.
	menu add: 'remove a variable...' action: #removeClassVar.
	^menu! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/11/2009 02:41'!
contextMenu
	| choice |
	choice _ header choice.
	(#basics = choice) ifTrue: [^nil ].

	(#(allClasses #currentPackageClasses) includes: choice) ifTrue: [
		^self classMenu].

	(#variables = choice) ifTrue: [
		(currentClass isMeta)
			ifTrue: [ ^self classVarMenu]
			ifFalse: [ ^self instanceVarMenu ]].

	(#instanceVars = choice) ifTrue: [^self instanceVarMenu].

	(#classVars = choice) ifTrue: [^self classVarMenu].

	(#globalVars = choice) ifTrue: [ ^self globalVarMenu].

	(#currentMessages = choice) ifTrue: [^self methodMenu].

	(#methodsClass = choice) ifTrue: [^nil ].! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 21:41'!
createClassVar
	|varName current|
	currentClass isMeta
		ifTrue: [current _ currentClass theNonMetaClass ]
		ifFalse: [current _ currentClass ].

	varName _ FillInTheBlank request: 'variable name:' initialAnswer: ''.
	varName isEmpty ifTrue: [^self].
	current addClassVarName: varName.
	self refresh		! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/16/2009 21:07'!
createGlobalVar
	|varName |

	varName _ FillInTheBlank request: 'variable name:' initialAnswer: ''.
	varName isEmpty ifTrue: [^self].
	varName _ SyntaxElementMorph classNameFrom: varName.
	(Smalltalk includesKey: varName asSymbol)
		ifTrue: [^self inform: varName, ' is already in use'].
	Smalltalk at: varName asSymbol put: nil.
	self refresh		! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/16/2009 21:08'!
createInstVar
	|varName current|
	currentClass isMeta
		ifTrue: [current _ currentClass theNonMetaClass ]
		ifFalse: [current _ currentClass ].

	varName _ FillInTheBlank request: 'variable name:' initialAnswer: ''.
	varName isEmpty ifTrue: [^self].
	varName _ SyntaxElementMorph objectNameFrom: varName.
	current addInstVarName: varName.
	self refresh		! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/16/2009 21:09'!
createSubclass
	|className categoryName current newClass|
	currentClass isMeta
		ifTrue: [current _ currentClass theNonMetaClass ]
		ifFalse: [current _ currentClass ].

	className _ FillInTheBlank request: 'class name:' initialAnswer: ''.
	className isEmpty ifTrue: [^self].
	categoryName _ FillInTheBlank request: 'category:' initialAnswer: 'UserObjects'.
	categoryName isEmpty ifTrue: [^self].

	className _ SyntaxElementMorph classNameFrom: className.

	newClass _ current subclass: className asSymbol
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: categoryName.

	editorFrame selectClass: newClass

		! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/7/2009 21:12'!
currentClass
	^currentClass! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/16/2009 12:48'!
currentMethodsClass
	
	| name |
	name _ palette contents submorphs last contents.
	(name endsWith: ' class') ifTrue: [
		name _ (name reversed copyFrom: 6 to: name size) reversed.
		name _ name reject: [:c| c = $ ].
		^(Smalltalk at: name asSymbol) class ].

	name _ name reject: [:c| c = $ ].
	^Smalltalk at: name asSymbol! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/6/2009 13:42'!
editorFrame: aMorph
	editorFrame _ aMorph! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 21:56'!
globalVarMenu
	|menu|
	menu _ MenuMorph new defaultTarget: self.
	menu add: 'add a global variable...' action: #createGlobalVar.
	menu add: 'remove a variable...' action: #removeGlobalVar.
	^menu! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 19:50'!
handlesMouseDown: evt
	"Return true if this morph wants to receive mouseDown events (i.e., mouseDown:, mouseMove:, mouseUp:). The default response is false; subclasses that implement mouse messages should override this to return true." 

	^ true! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/7/2009 20:42'!
initialize
	super initialize.
	currentClass _ Object.
	self extent: 180 @ 400.
	self buildPanes.
	self arrangePanes.
	self refresh! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 21:26'!
instanceVarMenu
	|menu|
	menu _ MenuMorph new defaultTarget: self.
	menu add: 'add an instance variable...' action: #createInstVar.
	menu add: 'remove a variable...' action: #removeInstVar.
	^menu! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 20:45'!
invokeContextMenu
	|m |
	m _ self contextMenu.
	m ifNotNil: [m popUpNearHand]! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 22:13'!
methodMenu
	|menu|
	menu _ MenuMorph new defaultTarget: self.
	menu add: 'remove a method...' action: #removeMethod.
	^menu! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 19:51'!
mouseDown: evt
	"Handle a mouse down event."

	evt rightButtonPressed
		ifTrue: [ ^self invokeContextMenu].
! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/9/2009 02:02'!
refresh
	| choice |
	choice _ header choice.
	(#basics = choice) ifTrue: [
		palette contents: ElementPaletteMorph basics ].

	(#allClasses = choice) ifTrue: [
		palette contents: ElementPaletteMorph classes ].

	(#currentPackageClasses = choice) ifTrue: [
		(currentClass isKindOf: Metaclass)
			ifTrue: [ palette contents: (ElementPaletteMorph currentPackageClasses: currentClass allInstances first) ]
			ifFalse: [palette contents: (ElementPaletteMorph currentPackageClasses: currentClass) ]].


	(#variables = choice) ifTrue: [
		(currentClass isKindOf: Metaclass)
			ifTrue: [ palette contents: (ElementPaletteMorph classVarsFor: currentClass allInstances first) ]
			ifFalse: [palette contents: (ElementPaletteMorph instVarsFor: currentClass) ]].

	(#instanceVars = choice) ifTrue: [
		(currentClass isKindOf: Metaclass)
			ifTrue: [palette contents: (ElementPaletteMorph instVarsFor: currentClass allInstances first) ]
			ifFalse: [palette contents: (ElementPaletteMorph instVarsFor: currentClass) ]].

	(#classVars = choice) ifTrue: [
		(currentClass isKindOf: Metaclass)
			ifTrue: [palette contents: (ElementPaletteMorph classVarsFor: currentClass allInstances first) ]
			ifFalse: [palette contents: (ElementPaletteMorph classVarsFor: currentClass) ]].

	(#globalVars = choice) ifTrue: [
		palette contents: (ElementPaletteMorph globals) ].



	(#currentMessages = choice) ifTrue: [
		palette contents: (ElementPaletteMorph messagesFor: currentClass) ].

	(#methodsClass = choice) ifTrue: [
		palette contents: (ElementPaletteMorph messagesFor: header methodsClass) ].! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 21:21'!
removeClass
	| class |
	class _ self selectClass.
	class ifNotNil: [
		class _ Smalltalk at: class.
		self removeClass: class ]! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 21:22'!
removeClass: aClass

		(editorFrame currentClass = aClass)
			ifTrue: [editorFrame selectClass: Object ].
		aClass removeFromSystem.
		self refresh ! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 21:43'!
removeClassVar
	| var |
	var _ self selectClassVar.
	var ifNotNil: [
		self currentClass removeClassVarName: var.
		self refresh ]! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 22:02'!
removeGlobalVar
	| var |
	var _ self selectGlobalVar.
	var ifNotNil: [
		Smalltalk removeKey: var.
		self refresh ]! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 21:35'!
removeInstVar
	| var |
	var _ self selectInstVar.
	var ifNotNil: [
		self currentClass removeInstVarName: var.
		self refresh ]! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 22:15'!
removeMethod
	| sel |
	sel _ self selectSelector.
	sel ifNotNil: [
		self currentClass removeSelector: sel.
		self refresh ]! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 21:07'!
selectClass
	^(CustomMenu selections: Smalltalk classNames) startUp
! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 21:44'!
selectClassVar
	^(CustomMenu selections: currentClass classVarNames asSortedCollection) startUp
! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 22:03'!
selectGlobalVar
	^(CustomMenu selections: (Smalltalk keys reject: [:key| Smalltalk classNames includes: key]) asSortedCollection) startUp
! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 21:34'!
selectInstVar
	^(CustomMenu selections: currentClass instVarNames) startUp
! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/10/2009 22:16'!
selectSelector
	^(CustomMenu selections: currentClass selectors asSortedCollection) startUp
! !

!ElementsPaletteFrameMorph methodsFor: 'as yet unclassified' stamp: 'jens 2/5/2009 20:29'!
wantsDroppedMorph: aMorph event: evt
	"Return true if the receiver wishes to accept the given morph, which is being dropped by a hand in response to the given event. The default implementation returns false.
NOTE: the event is assumed to be in global (world) coordinates."

	^aMorph isKindOf: SyntaxElementMorph! !
