BorderedMorph subclass: #ScrollFrameMorph
	instanceVariableNames: 'contents hScrollbar vScrollbar cornerMorph hbarInset vbarInset contentsChanged growthFraction edge delay hasWaited '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Scratch-UI-Support'!
!ScrollFrameMorph commentStamp: '<historical>' prior: 0!
I am a frame for scrollable contents. I can independently show or hide my horizontal and vertical scrollbars.


!


!ScrollFrameMorph methodsFor: 'initialization' stamp: 'jm 3/3/2008 22:30'!
initialize

	super initialize.
	self color: Color red.
	hbarInset _ 12.
	vbarInset _ 12.
	contentsChanged _ false.
	growthFraction _ 0.6.  "amount to grow when contents do not fit as a fraction of the current size"

	contents _ PasteUpMorph new
		color: Color red;
		borderWidth: 0;
		enableDragNDrop: true.

	hScrollbar _ ScrollBar new
		model: self;
		setValueSelector: #hScrollRelative:;
		hasMenuButton: false.
	vScrollbar _ ScrollBar new
		model: self;
		setValueSelector: #vScrollRelative:;
		hasMenuButton: false.
	cornerMorph _ BorderedMorph new
		borderRaised; borderWidth: 2; color: hScrollbar color.

	contents position: self position + borderWidth.
	self addMorph: contents.
	self addMorph: hScrollbar.
	self addMorph: vScrollbar.
	self addMorph: cornerMorph.
	self extent: 160@120.
! !


!ScrollFrameMorph methodsFor: 'accessing' stamp: 'jm 8/2/2003 14:34'!
contents
	"Answer my contents morph."

	^ contents
! !

!ScrollFrameMorph methodsFor: 'accessing' stamp: 'jm 8/3/2003 10:48'!
contents: aMorph
	"Replace my contents morph. The argument can be any morph. Typically it is a BorderedMorph or a PasteUpMorph."

	contents ifNotNil: [contents delete].
	contents _ aMorph.
	contents position: self topLeft + borderWidth.
	self addMorphBack: contents.
	self extent: self extent.
! !

!ScrollFrameMorph methodsFor: 'accessing' stamp: 'jm 3/3/2008 22:32'!
growthFraction: aNumber
	"Set the amount to grow when contents do not fit, expressed as a fraction of the current size. For example, 0.5 makes the contains pane 50% larger if any submorph extends over the edge."

	growthFraction _ aNumber max: 0.0.
! !

!ScrollFrameMorph methodsFor: 'accessing' stamp: 'ee 5/1/2008 10:20'!
hBarInset: anInteger

	hbarInset _ anInteger.
! !

!ScrollFrameMorph methodsFor: 'accessing' stamp: 'ee 5/1/2008 10:19'!
vBarInset: anInteger

	vbarInset _ anInteger.
! !


!ScrollFrameMorph methodsFor: 'scrolling' stamp: 'jm 9/9/2003 10:52'!
hScrollPixels
	"Answer the current horizontal scroll offset in pixels."

	^ (self left + borderWidth) - contents left

! !

!ScrollFrameMorph methodsFor: 'scrolling' stamp: 'jm 9/10/2003 11:58'!
hScrollPixels: hOffset
	"Scroll to the given horizontal offset in pixels. Zero is scrolled to the left and increasing values scroll right."

	| delta maxOffset |
	delta _ (hOffset asInteger min: self maxScroll x) max: 0.
	contents left: ((self left + borderWidth) - delta) truncated.

	maxOffset _ self maxScroll x.
	maxOffset > 0 ifTrue: [hScrollbar value: self hScrollPixels / maxOffset].
! !

!ScrollFrameMorph methodsFor: 'scrolling' stamp: 'jm 8/2/2003 20:25'!
hScrollRelative: aFloat
	"Sent by the horizontal scrollbar. Scroll to the given relative postion between 0.0 and 1.0."

	self hScrollPixels: aFloat * self maxScroll x.
! !

!ScrollFrameMorph methodsFor: 'scrolling' stamp: 'jm 8/2/2003 20:25'!
maxScroll
	"Answer a point representing the maximum horizontal and vertical scroll offsets in pixels."

	^ contents extent - self visibleExtent
! !

!ScrollFrameMorph methodsFor: 'scrolling' stamp: 'jm 6/23/2005 16:47'!
scrollSubmorphIntoView: aMorph

	| offset |
	(contents submorphs includes: aMorph) ifFalse: [^ self].
	(self bounds containsRect: aMorph fullBounds) ifTrue: [^ self].

	(aMorph fullBounds bottom > self bottom) ifTrue: [
		offset _ aMorph fullBounds bottom - contents top.
		contents top: (self bottom - 3) - offset].

	(aMorph fullBounds top < self top) ifTrue: [
		offset _ aMorph fullBounds top - contents top.
		contents top: (self top + 3) - offset].

	self updateScrollbars.
! !

!ScrollFrameMorph methodsFor: 'scrolling' stamp: 'jm 9/9/2003 10:52'!
vScrollPixels
	"Answer the current vertical scroll offset in pixels."

	^ (self top + borderWidth) - contents top

! !

!ScrollFrameMorph methodsFor: 'scrolling' stamp: 'jm 9/10/2003 11:58'!
vScrollPixels: vOffset
	"Scroll to the given vertical offset in pixels. Zero is scrolled to the top and increasing values scroll down."

	| delta maxOffset |
	delta _ (vOffset asInteger min: self maxScroll y) max: 0.
	contents top: ((self top + borderWidth) - delta) truncated.

	maxOffset _ self maxScroll y.
	maxOffset > 0 ifTrue: [vScrollbar value: self vScrollPixels / maxOffset].
! !

!ScrollFrameMorph methodsFor: 'scrolling' stamp: 'jm 8/2/2003 15:25'!
vScrollRelative: aFloat
	"Sent by the vertical scrollbar. Scroll to the given relative postion between 0.0 and 1.0."

	self vScrollPixels: aFloat * self maxScroll y.
! !


!ScrollFrameMorph methodsFor: 'scrollbar visibility' stamp: 'jm 8/2/2003 14:52'!
showHorizontalScrollbar: aBoolean
	"Show or hide my horizontal scrollbar."

	aBoolean
		ifTrue: [
			self addMorph: hScrollbar.
			vScrollbar owner = self ifTrue: [self addMorph: cornerMorph]]
		ifFalse: [
			hScrollbar delete.
			cornerMorph delete].

	self extent: self extent.
! !

!ScrollFrameMorph methodsFor: 'scrollbar visibility' stamp: 'jm 8/2/2003 14:52'!
showVerticalScrollbar: aBoolean
	"Show or hide my vertical scrollbar."

	aBoolean
		ifTrue: [
			self addMorph: vScrollbar.
			hScrollbar owner = self ifTrue: [self addMorph: cornerMorph]]
		ifFalse: [
			vScrollbar delete.
			cornerMorph delete].

	self extent: self extent.
! !


!ScrollFrameMorph methodsFor: 'drawing' stamp: 'jm 2/1/2005 12:01'!
drawSubmorphsOn: aCanvas
	"If my contents has changed, fix it's extent and update my scrollbar ranges. Clip submorph drawing to my bounds."

	contentsChanged ifTrue: [
		self updateContentsExtent.
		self updateScrollbars.
		contentsChanged _ false].

	super drawSubmorphsOn: (aCanvas copyClipRect: self innerBounds).
! !


!ScrollFrameMorph methodsFor: 'layout' stamp: 'jm 8/2/2003 14:26'!
hResizing

	^ #spaceFill
! !

!ScrollFrameMorph methodsFor: 'layout' stamp: 'jm 8/2/2003 14:24'!
isAlignmentMorph
	"Answer true, since I can be laid out as if I were an AlignmentMorph. I pretend to be an AlignmentMorph so that I can be resized when I'm inside another AlignmentMorph."

	^ true
! !

!ScrollFrameMorph methodsFor: 'layout' stamp: 'jm 8/2/2003 14:22'!
layoutInWidth: w height: h
	"Resize myself to the given width and height. Called during when I am in an AlignmentMorph."

	self extent: w@h.
! !

!ScrollFrameMorph methodsFor: 'layout' stamp: 'jm 7/21/2003 21:06'!
minHeight

	^ 45
! !

!ScrollFrameMorph methodsFor: 'layout' stamp: 'jm 7/21/2003 21:06'!
minWidth

	^ 60
! !

!ScrollFrameMorph methodsFor: 'layout' stamp: 'jm 8/2/2003 14:26'!
vResizing

	^ #spaceFill
! !


!ScrollFrameMorph methodsFor: 'geometry' stamp: 'jm 2/23/2005 17:48'!
extent: aPoint
	"After setting my size, position and size my scrollbars and grow box. Also update my contents and scrollbar ranges."

	| inner w h |
	super extent: (aPoint truncated max: self minWidth@self minHeight).
	inner _ self innerBounds.

	w _ inner width.
	vScrollbar owner = self ifTrue: [w _ w - vbarInset].
	hScrollbar position: inner left@(inner bottom - hbarInset).
	hScrollbar extent: w@hbarInset.

	h _ inner height.
	hScrollbar owner = self ifTrue: [h _ h - hbarInset].
	vScrollbar position: (inner right - vbarInset)@inner top.
	vScrollbar extent: vbarInset@h.

	cornerMorph position: (inner bottomRight - (vbarInset@hbarInset)).
	cornerMorph extent: vbarInset@hbarInset.

	self updateContentsExtent.
	self updateScrollbars.
! !

!ScrollFrameMorph methodsFor: 'geometry' stamp: 'jm 7/24/2003 18:15'!
fullBounds
	"Overridden to clip submorph hit detection to my bounds."

	^ bounds
! !

!ScrollFrameMorph methodsFor: 'geometry' stamp: 'jm 8/3/2003 09:11'!
invalidRect: damageRect
	"Clip damage reports to my bounds, since drawing is clipped to my bounds."

	| r |
	r _ damageRect intersect: self bounds.
	(r width > 0 and: [r height > 0]) ifTrue: [super invalidRect: r].
! !

!ScrollFrameMorph methodsFor: 'geometry' stamp: 'jm 8/2/2003 14:31'!
layoutChanged
	"If my contents morph's layout has changed, record that fact."

	super layoutChanged.
	contents mayNeedLayout ifTrue: [contentsChanged _ true].
! !


!ScrollFrameMorph methodsFor: 'private' stamp: 'jm 8/2/2003 20:02'!
contentsBounds
	"Answer a rectangle that encloses the bounds of all my submorphs."

	| offset r |
	offset _ contents topLeft negated.
	r _ 0@0 extent: 1@1.
	contents submorphsDo: [:m |
		r _ r quickMerge: (m fullBounds translateBy: offset) truncated].
	^ r
! !

!ScrollFrameMorph methodsFor: 'private' stamp: 'ee 3/8/2008 16:28'!
updateContentsExtent
	"Make sure my content morph is large enough to both enclose all it's submorphs and to fill me. Adjust the scroll offsets if my bounds have shrunk."

	| r delta scrolledOff w h visible |
	r _ self contentsBounds.
	((r left < 0) or: [r top < 0]) ifTrue: [
		"contents has morphs that stick out over its top or left edge"
		delta _ (r topLeft truncated min: (0@0)) negated.
		contents submorphsDo: [:m | m position: m position + delta]].  "move all submorphs by delta"

	visible _ self visibleExtent - (8@20).
	w _ visible x. r width > w ifTrue: [w _ (r width + (growthFraction * visible x)) rounded].
	h _ visible y. r height > h ifTrue: [h _ (r height + (growthFraction * visible y))  rounded].

	scrolledOff _ ((self topLeft + borderWidth) - contents position) max: 0@0.
	contents extent: ((w@h) max: (self visibleExtent + scrolledOff)).
	contentsChanged _ false.
! !

!ScrollFrameMorph methodsFor: 'private' stamp: 'jens 7/14/2010 23:32'!
updateScrollbars
	"Update my scrollbars based on my current contents."

	| visibleExtent currentScroll maxScroll |
	contents ifNil: [^ self].  "not fully initialized yet"

	visibleExtent _ self visibleExtent.
	hScrollbar percentVisible: (visibleExtent x asFloat / contents width).

	contents height = 0 
		ifTrue: [ self showVerticalScrollbar: false]
		ifFalse: [ vScrollbar percentVisible: (visibleExtent y asFloat / contents height)].

	currentScroll _ (self topLeft + borderWidth) - contents position.
	contents width > visibleExtent x
		ifTrue: [
			maxScroll _ contents width - visibleExtent x.
			self showHorizontalScrollbar: true.
			hScrollbar
				scrollDelta: 4 / maxScroll asFloat 
				pageDelta: (visibleExtent x - 40) / maxScroll asFloat.
			hScrollbar value: currentScroll x / maxScroll]
		ifFalse: [
			self showHorizontalScrollbar: false.
			hScrollbar value: 0].

	contents height > visibleExtent y
		ifTrue: [
			maxScroll _ contents height - visibleExtent y.
			self showVerticalScrollbar: true.
			vScrollbar
				scrollDelta: 4.0 / maxScroll asFloat 
				pageDelta: (visibleExtent y - 40) / maxScroll asFloat.
			vScrollbar value: currentScroll y / maxScroll]
		ifFalse: [
			self showVerticalScrollbar: false.
			vScrollbar value: 0].
! !

!ScrollFrameMorph methodsFor: 'private' stamp: 'jm 4/6/2005 11:48'!
visibleExtent
	"Answer the extent of my visible area. That is, the area within my borders minus the space used for scrollbars."

	| clipToScrollBarEdge visibleW visibleH |
	clipToScrollBarEdge _ false.
	clipToScrollBarEdge ifFalse: [^ self extent - (2 * borderWidth)].

	visibleW _ self width - (2 * borderWidth).
	vScrollbar owner = self ifTrue: [visibleW _ visibleW - vbarInset].
	visibleH _ self height - (2 * borderWidth).
	hScrollbar owner = self ifTrue: [visibleH _ visibleH - hbarInset].
	^ visibleW@visibleH
! !


!ScrollFrameMorph methodsFor: 'autoscrolling' stamp: 'jens 8/20/2008 14:43'!
abortDelay
	"private"

	delay ifNotNil: [
		delay terminate.
		delay _ nil ]! !

!ScrollFrameMorph methodsFor: 'autoscrolling' stamp: 'jens 8/20/2008 16:24'!
nearestEdge: aPoint
	"private - answer a symbol indicating which edge aPoint is near to, nil if none is close"

	| cordon |

	"distance to the nearest edge within which automatic
	scrolling will be triggered"
	cordon _ 55.

	(self findA: ScratchScrollBar) ifNil: [^nil ].
	(self containsPoint: aPoint) ifFalse: [^nil ].

	aPoint y < (self top + cordon) ifTrue: [^#top ].
	aPoint y > (self bottom - cordon) ifTrue: [^#bottom ].
	aPoint x < (self left + cordon) ifTrue: [^#left ].
	aPoint x > (self right - cordon) ifTrue: [^#right ].

	^nil! !

!ScrollFrameMorph methodsFor: 'autoscrolling' stamp: 'jens 8/20/2008 15:46'!
scrollTowardsEdge: aSymbol
	"private - autoscrolling"

	|step|

	"the number of pixels to be scrolled in one pass:"
	step _ 10.

	"lazy initialization"
	hasWaited ifNil: [hasWaited _ false ].

	edge = aSymbol 	ifFalse: [ 
		self abortDelay.
		hasWaited _ false.
		edge _ aSymbol.
		edge ifNil: [^self ].
		^delay _ [(Delay forSeconds: 1) wait.
			hasWaited _ true.
			self scrollTowardsEdge: (self nearestEdge: World activeHand cursorPoint)] bg].

	hasWaited ifFalse: [	^self].

	#left = aSymbol ifTrue: [ 
		^self hScrollPixels: self hScrollPixels - step; 
		abortDelay ].
	#top = aSymbol ifTrue: [ 
		^self vScrollPixels: self vScrollPixels - step; 
		abortDelay  ].
	#right = aSymbol ifTrue: [ 
		^self hScrollPixels: self hScrollPixels + step; 
		abortDelay  ].
	#bottom = aSymbol ifTrue: [ 
		^self vScrollPixels: self vScrollPixels + step; 
		abortDelay  ]

! !


!ScrollFrameMorph methodsFor: 'event handling' stamp: 'jens 8/20/2008 14:41'!
handlesMouseOverDragging: evt
	^true! !

!ScrollFrameMorph methodsFor: 'event handling' stamp: 'jens 8/20/2008 14:41'!
mouseEnterDragging: evt
	"Handle a mouseEnterDragging event. The mouse just entered my bounds with a button pressed or laden with submorphs."

	self abortDelay.
	hasWaited _ false.
	edge _ nil.
! !

!ScrollFrameMorph methodsFor: 'event handling' stamp: 'jens 8/20/2008 15:47'!
mouseMoveDragging: evt
	"trigger autoscrolling"

	(evt hand submorphs select: [:m |
		contents wantsDroppedMorph: m event: evt ])
	isEmpty ifFalse: [ 
		self scrollTowardsEdge: (self nearestEdge: evt cursorPoint)]
! !
