ArgMorph subclass: #ExpressionArgMorph
	instanceVariableNames: 'isNumber isText '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Scratch-Blocks'!
!ExpressionArgMorph commentStamp: '<historical>' prior: 0!
I represent a literal value such as a number or string. I can be edited.
!


!ExpressionArgMorph methodsFor: 'initialization' stamp: 'jens 1/20/2010 19:37'!
initialize

	super initialize.
	isNumber _ true.
	isText _ false.
	self borderWidth: 1.
	self borderInset.
	self color: (Color gray: 0.95).
	self extent: 25@12.

	"To allow for editing in place."
	labelMorph _ StringFieldMorph new
		forExpressionArg;
		doResizing: true;
		font: (ScratchFrameMorph getFont: #Arg);
		color: Color transparent.
	self addMorph: labelMorph.
	self numExpression: '10'.
! !


!ExpressionArgMorph methodsFor: 'accessing' stamp: 'jens 3/3/2010 21:43'!
defaultValue: anObject

	(isNumber or: [anObject isNumber]) ifTrue: [^self numExpression: anObject].
	(anObject isKindOf: String) ifTrue: [self stringExpression: anObject].
! !

!ExpressionArgMorph methodsFor: 'accessing' stamp: 'jens 1/22/2010 02:17'!
defaultValueFromSpec: anObject
	"Like defaultValue:, but '-' in a block spec creates an empty number expression."

	'-' asUTF8 = anObject 
		ifTrue: [self numExpression: ' ']  "empty, evaluates to zero"
		ifFalse: [self defaultValue: anObject].

! !

!ExpressionArgMorph methodsFor: 'accessing' stamp: 'jens 3/3/2010 21:55'!
numExpression: aNumber

	isNumber _ true.
	labelMorph isNumeric: true.

	aNumber isFloat
		ifTrue: [labelMorph contents: aNumber printStringNoExponent]
		ifFalse: [labelMorph contents: aNumber asString].
	self fixArgLayout.
! !

!ExpressionArgMorph methodsFor: 'accessing' stamp: 'jens 2/19/2010 00:25'!
stringExpression: aString

	isNumber _ false.
	labelMorph isNumeric: false.
	labelMorph contents: aString.
	self fixArgLayout.
! !


!ExpressionArgMorph methodsFor: 'queries' stamp: 'jm 7/7/2004 00:24'!
acceptsDroppedReporters

	^ true
! !

!ExpressionArgMorph methodsFor: 'queries' stamp: 'jm 11/28/2006 13:14'!
acceptsTypeOf: aBlockMorph
	"Answer true if I can accept a dropped reporter of the given type."

	aBlockMorph isReporter ifFalse: [^ false].
	^ isNumber not or: [aBlockMorph isBooleanReporter not]
! !

!ExpressionArgMorph methodsFor: 'queries' stamp: 'jens 2/21/2010 23:41'!
isEmptySlot

	isText ifTrue: [
		^ labelMorph savedContents = '' asUTF8].
	^ (Array with: '' asUTF8 with: ' ' asUTF8) includes: labelMorph savedContents
! !


!ExpressionArgMorph methodsFor: 'drawing' stamp: 'jm 3/29/2005 20:18'!
drawOn: aCanvas 

	| darkerC right topY bottomY radius xInset c |
	isNumber ifFalse: [^ super drawOn: aCanvas].

	darkerC _ Color gray.
	right _ self width.
	topY _ bottomY _ radius _ self height // 2.
	self height even ifTrue: [topY _ bottomY - 1].
	[topY >= 0] whileTrue: [
		xInset _ radius - (radius squared - (radius - topY - 1) squared) sqrt rounded.

		self drawHLineFrom: xInset to: (xInset + 1) y: topY color: darkerC on: aCanvas.
		c _ (topY < 1) ifTrue: [darkerC] ifFalse: [Color white].
		self drawHLineFrom: xInset + 1 to: right - (xInset + 1) y: topY color: c on: aCanvas.
		self drawHLineFrom: (right - (xInset + 1)) to: (right - xInset) y: topY color: darkerC on: aCanvas.

		self drawHLineFrom: xInset to: right - xInset y: bottomY color: Color white on: aCanvas.

		xInset = 0 ifTrue: [
			self drawHLineFrom: xInset + 1 to: xInset + 2 y: topY color: Color white on: aCanvas.
			self drawHLineFrom: xInset to: xInset + 1 y: bottomY color: darkerC on: aCanvas.
			self drawHLineFrom: (right - (xInset + 1)) to: (right - xInset) y: bottomY color: darkerC on: aCanvas].

		bottomY _ bottomY + 1.
		topY _ topY - 1].

! !

!ExpressionArgMorph methodsFor: 'drawing' stamp: 'jm 5/8/2008 12:14'!
isNumberExpression

	^ isNumber
! !

!ExpressionArgMorph methodsFor: 'drawing' stamp: 'jens 1/5/2010 21:02'!
isRectangular

	^ isNumber not
! !


!ExpressionArgMorph methodsFor: 'evaluation' stamp: 'jens 6/15/2010 23:56'!
evaluate
	"Answer the result of evaluating my expression in the context of the given ScratchProcess."

	| expr |

	(self isEmptySlot and: [implicitValue notNil])
		ifTrue: [^implicitValue].

	expr _ labelMorph savedContents.
	isNumber
		ifTrue: [^ expr asNumberNoError]
		ifFalse: [^ expr].  
! !


!ExpressionArgMorph methodsFor: 'object i/o' stamp: 'jm 9/24/2003 18:19'!
fieldsVersion

	^ 1
! !

!ExpressionArgMorph methodsFor: 'object i/o' stamp: 'jm 6/21/2004 17:57'!
initFieldsFrom: anObjStream version: classVersion

	super initFieldsFrom: anObjStream version: classVersion.
	self initFieldsNamed: #(
		isNumber
	) from: anObjStream.
! !

!ExpressionArgMorph methodsFor: 'object i/o' stamp: 'jm 6/21/2004 17:57'!
storeFieldsOn: anObjStream

	super storeFieldsOn: anObjStream.
	self storeFieldsNamed: #(
		isNumber
	) on: anObjStream.
! !


!ExpressionArgMorph methodsFor: 'private' stamp: 'jens 1/20/2010 19:49'!
fixArgLayout

	| top xInset extraW |
	top _ self top.
	isNumber
		ifTrue: [
			self extent: labelMorph extent + (5 @ 0).
			"tiny vertical adjustment to account for differences in string rendering"
			Smalltalk isWindows ifFalse: [top _ top + 1].
			"for tall fonts, add extra width"
			extraW _ (self height - (1.3 * self width) rounded) max: 0.
			extraW > 0 ifTrue: [self width: self width + extraW].
			xInset _ 4 + (extraW // 2)]
		ifFalse: [
			isText
				ifTrue: [
					self extent: labelMorph extent + (2 @ 0).
					self width: (self width max: 25).
					xInset _ 3]
				ifFalse: [
					self extent: labelMorph extent + (2 @ 0).
					xInset _ 3]].

	ScratchTranslator isRTL
		ifTrue: [labelMorph position: (self right - (xInset + labelMorph width)) @ top]
		ifFalse: [labelMorph position: (self left + xInset) @ top].
! !


!ExpressionArgMorph methodsFor: 'BYOB' stamp: 'jens 6/15/2010 22:38'!
getExpression

	^ labelMorph savedContents! !

!ExpressionArgMorph methodsFor: 'BYOB' stamp: 'jens 1/13/2010 21:19'!
isNumberExpression: bool

	isNumber _ bool
! !

!ExpressionArgMorph methodsFor: 'BYOB' stamp: 'jens 1/20/2010 19:37'!
isText

	^ isText ifNil: [isText _ false ]! !

!ExpressionArgMorph methodsFor: 'BYOB' stamp: 'jens 1/20/2010 19:36'!
isText: bool

	isText _ bool.
	isNumber _ bool not
! !
