HatBlockMorph subclass: #ProcedureHatBlockMorph
	instanceVariableNames: 'argMorph lastState '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Scratch-BYOB'!

!ProcedureHatBlockMorph methodsFor: 'initialization' stamp: 'jens 1/12/2010 09:06'!
initialize

	super initialize.
	argMorph _ ExpressionArgMorph new.

	self removeAllMorphs.
	self addMorphBack: argMorph.
! !


!ProcedureHatBlockMorph methodsFor: 'other' stamp: 'jens 6/22/2010 20:41'!
argMorph

	self nonControlFlowSubmorphs do: [:m |
		(m isKindOf: ArgMorph) ifTrue: [^ m].
		((m isKindOf: BlockMorph) and: [m isCustomBlock]) ifTrue: [^ m]].
	^ nil
! !

!ProcedureHatBlockMorph methodsFor: 'other' stamp: 'jens 1/12/2010 09:04'!
argMorphToReplace: aMorph
	"Answer a new argument morph to be used to replace the given morph."

	aMorph ~= argMorph ifTrue: [^ nil].
	^ BooleanArgMorph new
! !

!ProcedureHatBlockMorph methodsFor: 'other' stamp: 'jens 1/12/2010 09:04'!
asBlockTuple
	"Answer a tuple (Array) describing this block and its arguments."

	| argTuple |
	argTuple _ (argMorph isKindOf: ArgMorph)
		ifTrue: [argMorph evaluate]
		ifFalse: [argMorph asBlockTuple].

	^ Array with: #byob with: '' with: self class name with: argTuple
! !

!ProcedureHatBlockMorph methodsFor: 'other' stamp: 'jens 1/12/2010 09:04'!
evaluateCondition
	"Answer true if my condition is true."

	| arg |
	arg _ submorphs detect: [:m | m isKindOf: ReporterBlockMorph] ifNone: [^ false].
	^ arg evaluate
! !

!ProcedureHatBlockMorph methodsFor: 'other' stamp: 'jens 1/12/2010 09:04'!
fixBlockLayout
	"Update the positions of my submorphs."

	| x centerY nextB |
	argMorph ifNil: [^ self].  "happens during initialization"

	blockLayoutNeeded ifFalse: [^ self].
	cachedForm _ nil.
	cachedFeedbackForm _ nil.

	((argMorph isKindOf: ArgMorph) and: [argMorph owner = self])
		ifTrue: [argMorph fixArgLayout].
	self height: 26 + argMorph height.

	x _ self left + 8.
	centerY _ bounds center y + 5.
	self nonControlFlowSubmorphs do: [:m |
		m position: x@(centerY - (m height // 2)).
		x _ x + m width + 5].

	self width: ((x + 4 - self left) max: self hatTopForm width).

	(nextB _ self nextBlock) ifNotNil: [
		nextB top: self bottom - 4.
		nextB fixBlockLayout].
! !

!ProcedureHatBlockMorph methodsFor: 'other' stamp: 'jens 1/12/2010 09:04'!
helpScreenName
	"Answer the name of the help screen for this hat block, or nil if no help is available."

	^ 'whenHat'
! !

!ProcedureHatBlockMorph methodsFor: 'other' stamp: 'jens 1/12/2010 09:04'!
nextBlock
	"Answer the block following myself in the block sequence or nil if I am the last block in the sequence. Each block is a submorph of the previous block."

	^ self submorphs
		detect: [:m | (m isKindOf: BlockMorph) and: [m ~= argMorph]]
		ifNone: [nil]
! !

!ProcedureHatBlockMorph methodsFor: 'other' stamp: 'jens 1/12/2010 09:04'!
printHatNameOn: aStream
	"Append a human-readable string for this hat block's name to the given stream."

	aStream nextPutAll: 'when '.
	self printCodeSubmorph: argMorph on: aStream.
	aStream nextPutAll: ' becomes true'; cr.
! !

!ProcedureHatBlockMorph methodsFor: 'other' stamp: 'jens 1/12/2010 09:04'!
replaceArgMorph: oldMorph by: newMorph
	"Replace oldMorph with newMorph. This is done when dropping a reporter block onto my argument."

	argMorph _ newMorph.
	self replaceSubmorph: oldMorph by: newMorph.
	self layoutChanged.
! !

!ProcedureHatBlockMorph methodsFor: 'other' stamp: 'jens 1/12/2010 09:04'!
triggered
	"Answer true if my condition has just become true."

	| condition |
	condition _ self evaluateCondition.
	condition & lastState not ifTrue: [lastState _ true. ^ true].  "trigger on false->true transition"
	lastState _ condition.
	^ false
! !


!ProcedureHatBlockMorph methodsFor: 'as yet unclassified' stamp: 'jens 1/21/2010 20:54'!
handlesMouseOver: evt

	^ self argMorph isKindOf: BlockMorph
! !

!ProcedureHatBlockMorph methodsFor: 'as yet unclassified' stamp: 'jens 1/21/2010 20:54'!
handlesMouseOverDragging: evt

	^  self argMorph isKindOf: BlockMorph
! !

!ProcedureHatBlockMorph methodsFor: 'as yet unclassified' stamp: 'jens 1/12/2010 10:08'!
mouseDown: evt

	argMorph editSpec! !

!ProcedureHatBlockMorph methodsFor: 'as yet unclassified' stamp: 'jens 4/15/2010 23:39'!
mouseEnter: evt

	self argMorph showEdgePlaceHolders! !

!ProcedureHatBlockMorph methodsFor: 'as yet unclassified' stamp: 'jens 1/21/2010 20:54'!
mouseEnterDragging: evt

	"self argMorph showArgumentPlaceHolders"! !

!ProcedureHatBlockMorph methodsFor: 'as yet unclassified' stamp: 'jens 1/21/2010 20:14'!
mouseLeave: evt

		self argMorph hideArgumentPlaceHolders! !

!ProcedureHatBlockMorph methodsFor: 'as yet unclassified' stamp: 'jens 1/21/2010 20:10'!
mouseLeaveDragging: evt

	self argMorph hideArgumentPlaceHolders! !

!ProcedureHatBlockMorph methodsFor: 'as yet unclassified' stamp: 'jens 1/12/2010 09:16'!
rootForGrabOf: aMorph

	(owner isKindOf: ScratchScriptsMorph) ifTrue: [^nil].

	^ super rootForGrabOf: aMorph! !

!ProcedureHatBlockMorph methodsFor: 'as yet unclassified' stamp: 'jens 1/12/2010 09:11'!
template: aBlock
	self replaceArgMorph: self argMorph by: aBlock! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ProcedureHatBlockMorph class
	instanceVariableNames: ''!

!ProcedureHatBlockMorph class methodsFor: 'as yet unclassified' stamp: 'jens 1/12/2010 09:11'!
on: aBlockMorph
	^self new template: aBlockMorph! !
